@@ -10,16 +10,15 @@ from base64 import b64decode
 from datetime import datetime
 from flask import request, jsonify
 from sqlalchemy.exc import IntegrityError
-from sqlalchemy.orm import aliased
 
-from .. import defaultListHandler, defaultObjectHandler
+from .. import defaultObjectHandler, userQuery
 
 from services import Service
 
 from tools import formats
 from tools.config import Config
 from tools.constants import PropTags, PropTypes, ExchangeErrors, PrivateFIDs, Permissions
-from tools.misc import createMapping, loadPSO, GenericObject
+from tools.misc import loadPSO, GenericObject
 from tools.permissions import SystemAdminPermission, DomainAdminPermission, DomainAdminROPermission
 from tools.rop import nxTime, makeEidEx
 from tools.storage import setDirectoryOwner, setDirectoryPermission
@@ -37,26 +36,7 @@ from orm import DB
 @secure(requireDB=True)
 def getUsers(domainID):
     checkPermissions(DomainAdminROPermission(domainID))
-    from orm.users import Users, UserProperties
-    verbosity = int(request.args.get("level", 1))
-    query, limit, offset, count = defaultListHandler(Users, filters=(Users.domainID == domainID,), result="query")
-    sorts = request.args.getlist("sort")
-    for s in sorts:
-        sprop, sorder = s.split(",", 1) if "," in s else (s, "asc")
-        if hasattr(PropTags, sprop.upper()):
-            up = aliased(UserProperties)
-            query = query.join(up, (up.userID == Users.ID) & (up.tag == getattr(PropTags, sprop.upper())))\
-                         .order_by(up._propvalstr.desc() if sorder == "desc" else up._propvalstr.asc())
-    data = [user.todict(verbosity) for user in query.limit(limit).offset(offset).all()]
-    if verbosity < 2 and "properties" in request.args:
-        tags = [getattr(PropTags, prop.upper(), None) for prop in request.args["properties"].split(",")]
-        for user in data:
-            user["properties"] = {}
-        usermap = createMapping(data, lambda x: x["ID"])
-        properties = UserProperties.query.filter(UserProperties.userID.in_(usermap.keys()), UserProperties.tag.in_(tags)).all()
-        for prop in properties:
-            usermap[prop.userID]["properties"][prop.name] = prop.val
-    return jsonify(count=count, data=data)
+    return userQuery(domainID)
 
 
 @API.route(api.BaseRoute+"/domains/<int:domainID>/users", methods=["POST"])
@@ -106,9 +86,10 @@ def deleteUser(user, deleteChatUser):
         DB.session.commit()
     except Exception:
         return jsonify(message="Cannot delete user: Database commit failed."), 500
-    with Service("exmdb", errors=Service.SUPPRESS_INOP) as exmdb:
-        client = exmdb.user(userdata)
-        client.unloadStore()
+    if userdata.maildir:
+        with Service("exmdb", errors=Service.SUPPRESS_INOP) as exmdb:
+            client = exmdb.user(userdata)
+            client.unloadStore()
     if request.args.get("deleteFiles") == "true":
         shutil.rmtree(userdata.maildir, ignore_errors=True)
     return jsonify(message="isded")
@@ -166,6 +147,8 @@ def rdUserStoreProps(domainID, userID):
     user = Users.query.filter(Users.ID == userID, Users.domainID == domainID).first()
     if user is None:
         return jsonify(message="User not found"), 404
+    if not user.maildir:
+        return jsonify(message="User has no store"), 400
     props = [prop for prop in request.args.get("properties", "").split(",") if prop != ""]
     try:
         user.properties = {prop: None for prop in props}
@@ -260,6 +243,8 @@ def getUserSyncData(domainID, userID):
     user = Users.query.filter(Users.ID == userID, Users.domainID == domainID).first()
     if user is None:
         return jsonify(message="User not found"), 404
+    if not user.maildir:
+        return jsonify(message="User has no store"), 400
     with Service("exmdb") as exmdb:
         devices = {}
         client = exmdb.user(user)
@@ -380,6 +365,8 @@ def removeDevice(domainID, userID, deviceID):
     user = Users.query.filter(Users.ID == userID, Users.domainID == domainID).first()
     if user is None:
         return jsonify(message="User not found"), 404
+    if not user.maildir:
+        return jsonify(message="User has no store"), 400
     with Service("exmdb") as exmdb:
         client = exmdb.user(user)
         client.removeDevice(Config["sync"]["syncStateFolder"], deviceID)
@@ -396,6 +383,8 @@ def removeSyncStates(domainID, userID):
     user = Users.query.filter(Users.ID == userID, Users.domainID == domainID).first()
     if user is None:
         return jsonify(message="User not found"), 404
+    if not user.maildir:
+        return jsonify(message="User has no store"), 400
     with Service("exmdb") as exmdb:
         client = exmdb.user(user)
         client.removeSyncStates(Config["sync"]["syncStateFolder"])
@@ -410,6 +399,8 @@ def resyncDevice(domainID, userID, deviceID):
     user = Users.query.filter(Users.ID == userID, Users.domainID == domainID).first()
     if user is None:
         return jsonify(message="User not found"), 404
+    if not user.maildir:
+        return jsonify(message="User has no store"), 400
     with Service("exmdb") as exmdb:
         client = exmdb.user(user)
         res = client.resyncDevice(Config["sync"]["syncStateFolder"], deviceID, userID)
@@ -444,9 +435,9 @@ def setDeviceWipe(domainID, userID, deviceID):
         device = UserDevices(dict(userID=userID, deviceID=deviceID, status=2))
         DB.session.add(device)
         DB.session.flush()
-    device.status = 2
+    device.status = data.get("status", 2)
     DB.session.add(UserDeviceHistory(dict(userDeviceID=device.ID, time=datetime.utcnow(), remoteIP=request.remote_addr,
-                                          status=2)))
+                                          status=device.status)))
     DB.session.commit()
     return jsonify(message="Device wipe requested.")
 
@@ -475,13 +466,13 @@ def setUserStoreAccess(domainID, userID):
     user = Users.query.filter(Users.ID == userID, Users.domainID == domainID).first()
     if user is None:
         return jsonify(message="User not found"), 404
-    if user.maildir is None:
+    if not user.maildir:
         return jsonify(message="User has no store"), 400
     data = request.get_json(silent=True)
     if data is None or "username" not in data:
         return jsonify(message="Invalid data"), 400
     primary = Users.query.with_entities(Users.ID).filter(Users.username == data["username"],
-                                                         Users.domainID == domainID).first()
+                                                         Users.orgID == user.orgID).first()
     if primary is None:
         return jsonify(message="Could not find user to grant access to"), 404
     eid = makeEidEx(0, PrivateFIDs.IPMSUBTREE)
@@ -503,17 +494,17 @@ def setUserStoreAccessMulti(domainID, userID):
     user = Users.query.filter(Users.ID == userID, Users.domainID == domainID).first()
     if user is None:
         return jsonify(message="User not found"), 404
-    if user.maildir is None:
+    if not user.maildir:
         return jsonify(message="User has no store"), 400
     data = request.get_json(silent=True)
     if data is None or "usernames" not in data:
         return jsonify(message="Invalid data"), 400
-    primary = Users.query.filter(Users.username.in_(data["usernames"]), Users.domainID == domainID)\
-                         .with_entities(Users.ID).all()
+    primary = Users.query.filter(Users.username.in_(data["usernames"]), Users.orgID == user.orgID)\
+                         .with_entities(Users.ID, Users.username).all()
     eid = makeEidEx(0, PrivateFIDs.IPMSUBTREE)
     with Service("exmdb") as exmdb:
         client = exmdb.user(user)
-        res = client.setFolderMembers(eid, data["usernames"], Permissions.STOREOWNER)
+        res = client.setFolderMembers(eid, [user.username for user in primary], Permissions.STOREOWNER)
     if DB.minVersion(91):
         UserSecondaryStores.query.filter(UserSecondaryStores.secondaryID == user.ID).delete(synchronize_session=False)
         if len(primary):
@@ -531,7 +522,7 @@ def getUserStoreAccess(domainID, userID):
     user = Users.query.filter(Users.ID == userID, Users.domainID == domainID).first()
     if user is None:
         return jsonify(message="User not found"), 404
-    if user.maildir is None:
+    if not user.maildir:
         return jsonify(message="User has no store"), 400
     with Service("exmdb") as exmdb:
         client = exmdb.user(user)
@@ -549,7 +540,7 @@ def deleteUserStoreAccess(domainID, userID, username):
     user = Users.query.filter(Users.ID == userID, Users.domainID == domainID).first()
     if user is None:
         return jsonify(message="User not found"), 404
-    if user.maildir is None:
+    if not user.maildir:
         return jsonify(message="User has no store"), 400
     with Service("exmdb") as exmdb:
         client = exmdb.user(user)
