@@ -78,7 +78,7 @@ class OrgParam(DB.Base):
             return None
         plain = {entry.key: entry.value for entry in entries}
 
-        config = {"connection": {}, "users": {}}
+        config = {"connection": {}, "groups": {}, "users": {}}
         _addIfDef(config, "disabled", plain, "ldap_disabled", type=lambda x: x.lower() in ("true", "yes", "1"))
         _addIfDef(config["connection"], "server", plain, "ldap_uri")
         _addIfDef(config["connection"], "bindUser", plain, "ldap_binddn")
@@ -95,6 +95,10 @@ class OrgParam(DB.Base):
         _addIfDef(config["users"], "defaultQuota", plain, "ldap_user_default_quota", type=int)
         _addIfDef(config["users"], "templates", plain, "ldap_user_templates", all=True)
         _addIfDef(config["users"], "aliases", plain, "ldap_user_aliases")
+        _addIfDef(config["groups"], "groupaddr", plain, "ldap_group_addr")
+        _addIfDef(config["groups"], "groupfilter", plain, "ldap_group_filter")
+        _addIfDef(config["groups"], "groupname", plain, "ldap_group_name")
+        _addIfDef(config["groups"], "groupMemberAttr", plain, "ldap_group_memberof")
         if "ldap_user_attributes" in plain:
             config["users"]["attributes"] = {entry.split(" ", 1)[0]: entry.split(" ", 1)[1]
                                              for entry in plain.getall("ldap_user_attributes") if " " in entry}
@@ -127,6 +131,11 @@ class OrgParam(DB.Base):
             if "attributes" in config["users"] and config["users"]["attributes"]:
                 flat["ldap_user_attributes"] = ["{} {}".format(key, value)
                                                 for key, value in config["users"]["attributes"].items()]
+        if "groups" in config:
+            _addIfDef(flat, "ldap_group_addr", config["groups"], "groupaddr")
+            _addIfDef(flat, "ldap_group_filter", config["groups"], "groupfilter")
+            _addIfDef(flat, "ldap_group_name", config["groups"], "groupname")
+            _addIfDef(flat, "ldap_group_memberof", config["groups"], "groupMemberAttr")
         cls.query.filter(cls.orgID == orgID).delete(synchronize_session=False)
         for key, value in flat.items():
             DB.session.add(OrgParam(orgID, key, value))
@@ -193,7 +202,7 @@ class Domains(DataModel, DB.Base, NotifyTable):
                      ({"attr": "syncPolicy", "flags": "patch"},
                       {"attr": "chat", "flags": "patch"},
                       RefProp("homeserver", "homeserverID", flags="patch", filter="set", qopt=selectinload)),
-                     (Text("homedir", flags="hidden"),))
+                     (Text("homedir", match=False, flags="hidden"),))
 
     NORMAL = 0
     SUSPENDED = 1
@@ -303,7 +312,6 @@ class Domains(DataModel, DB.Base, NotifyTable):
                            synchronize_session=False)
 
     def purge(self, deleteFiles=False, printStatus=False):
-        from .classes import Classes, Hierarchy, Members
         from .misc import DBConf
         from .mlists import MLists, Associations, Specifieds
         from .roles import AdminRoles as AR, AdminRolePermissionRelation as ARPR
@@ -323,10 +331,6 @@ class Domains(DataModel, DB.Base, NotifyTable):
             if printStatus:
                 print("Done.")
         nosync = {"synchronize_session": False}
-        classes = Classes.query.filter(Classes.domainID == self.ID).with_entities(Classes.ID)
-        Hierarchy.query.filter(Hierarchy.childID.in_(classes) | Hierarchy.classID.in_(classes)).delete(**nosync)
-        Members.query.filter(Members.classID.in_(classes)).delete(**nosync)
-        Classes.query.filter(Classes.domainID == self.ID).delete(**nosync)
         mlists = MLists.query.filter(MLists.domainID == self.ID)
         Specifieds.query.filter(Specifieds.listID.in_(mlists.with_entities(MLists.ID))).delete(**nosync)
         Associations.query.filter(Associations.listID.in_(mlists.with_entities(MLists.ID))).delete(**nosync)
