@@ -141,7 +141,7 @@ class Users(DataModel, DB.Base, NotifyTable):
     lang = Column("lang", VARCHAR(32), nullable=False, server_default="")
     homeserverID = OptionalC(104, "0", Column("homeserver", TINYINT(unsigned=True), nullable=False, server_default="0"))
     _syncPolicy = OptionalC(76, "NULL", Column("sync_policy", TEXT))
-    _deprecated_maxSize = Column("max_size", INTEGER(10), nullable=False, default=0)
+    _deprecated_maxSize = OptionalC(-111, "0", Column("max_size", INTEGER(10), nullable=False, default=0))
     _deprecated_addressType = OptionalC(-86, "NULL", Column("address_type", TINYINT, nullable=False, server_default="0"))
     _deprecated_subType = OptionalC(-85, "NULL", Column("sub_type", TINYINT, nullable=False, server_default="0"))
     _deprecated_groupID = Column("group_id", INTEGER(10, unsigned=True), nullable=False, index=True, default=0)
@@ -158,13 +158,14 @@ class Users(DataModel, DB.Base, NotifyTable):
     forward = relationship("Forwards", uselist=False, cascade="all, delete-orphan", back_populates="user")
     homeserver = OptionalNC(104, None,
                             relationship("Servers", foreign_keys=homeserverID, primaryjoin="Users.homeserverID == Servers.ID"))
+    mlist = relationship("MLists", back_populates="user", uselist=False)
 
     _dictmapping_ = ((Id(), Text("username", flags="patch")),
                      (Id("domainID", flags="init"),
                       {"attr": "ldapID", "flags": "patch"},
                       Int("status", flags="patch"),
                       Text("altname", flags="patch")),
-                     (Text("lang", flags="patch"),
+                     (Text("lang", match=False, flags="patch"),
                       BoolP("pop3_imap", flags="patch"),
                       BoolP("smtp", flags="patch"),
                       BoolP("changePassword", flags="patch"),
@@ -178,13 +179,14 @@ class Users(DataModel, DB.Base, NotifyTable):
                       {"attr": "properties", "flags": "patch", "func": lambda p: p.namemap()},
                       RefProp("roles", qopt=selectinload),
                       RefProp("forward", flags="managed, patch"),
+                      RefProp("mlist", filter="set", flat="ID"),
                       {"attr": "syncPolicy", "flags": "patch"},
                       {"attr": "chat", "flags": "patch"},
                       {"attr": "chatAdmin", "flags": "patch"},
                       Int("orgID"),
                       RefProp("homeserver", "homeserverID", flags="patch", filter="set", qopt=selectinload)),
                      ({"attr": "password", "flags": "init, hidden"},
-                      Text("maildir", flags="hidden"),))
+                      Text("maildir", match=False, flags="hidden"),))
 
     USER_PRIVILEGE_POP3_IMAP = 1 << 0
     USER_PRIVILEGE_SMTP = 1 << 1
@@ -228,7 +230,7 @@ class Users(DataModel, DB.Base, NotifyTable):
         from tools.license import getLicense
         if "username" not in data:
             return "Missing username"
-        if data.get("status", 0) != Users.SHARED and Users.count() >= getLicense().users:
+        if data.get("status", 0) not in (Users.SHARED, Users.CONTACT) and Users.count() >= getLicense().users:
             return "License user limit exceeded"
         if "domainID" in data:
             domain = Domains.query.filter(Domains.ID == data.get("domainID")).first()
@@ -607,7 +609,7 @@ class Users(DataModel, DB.Base, NotifyTable):
     def delete(self, deleteChatUser=True):
         """Delete user from database.
 
-        Also cleans up entries in forwards, members and associations tables.
+        Also cleans up entries in forwards and associations tables.
 
         Parameters
         ----------
@@ -619,13 +621,12 @@ class Users(DataModel, DB.Base, NotifyTable):
         str
             Error message or None if successful.
         """
-        from .mlists import Associations
-        from .classes import Members
+        from .mlists import Associations, MLists
         if self.ID == 0:
             raise ValueError("Cannot delete superuser")
         Forwards.query.filter(Forwards.username == self.username).delete(synchronize_session=False)
-        Members.query.filter(Members.username == self.username).delete(synchronize_session=False)
         Associations.query.filter(Associations.username == self.username).delete(synchronize_session=False)
+        MLists.query.filter(MLists.listname == self.username).delete(synchronize_session=False)
         if self.chatID is not None:
             from services import Service
             with Service("chat", errors=Service.SUPPRESS_ALL) as chat:
@@ -907,8 +908,9 @@ class Fetchmail(DataModel, DB.Base):
                 fetchoptions += " sslCertCheck"
             if self.sslCertPath:
                 fetchoptions += "sslcertpath "+self.sslCertPath
-        if self.fetchall == 1:
-            fetchoptions += " fetchall"
+        if self.fetchall == 0:
+            fetchoptions += " no"
+        fetchoptions += " fetchall"
         fetchoptions += " keep" if self.keep == 1 else " nokeep"
         if self.extraOptions:
             fetchoptions += " "+self.extraOptions
@@ -990,7 +992,7 @@ class Forwards(DataModel, DB.Base):
 
 
 from .domains import Domains
-from . import misc, roles
+from . import misc, mlists, roles
 
 
 Users.NTregister()
